@@ -1,11 +1,12 @@
-/*C- 
+/* 
 //C- -------------------------------------------------------------------
 //C- DjVuLibre-3.5
 //C- Copyright (c) 2002  Leon Bottou and Yann Le Cun.
 //C- Copyright (c) 2001  AT&T
 //C-
 //C- This software is subject to, and may be distributed under, the
-//C- GNU General Public License, Version 2. The license should have
+//C- GNU General Public License, either Version 2 of the license,
+//C- or (at your option) any later version. The license should have
 //C- accompanied the software or you may obtain a copy of the license
 //C- from the Free Software Foundation at http://www.fsf.org .
 //C-
@@ -14,10 +15,10 @@
 //C- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //C- GNU General Public License for more details.
 //C- 
-//C- DjVuLibre-3.5 is derived from the DjVu(r) Reference Library
-//C- distributed by Lizardtech Software.  On July 19th 2002, Lizardtech 
-//C- Software authorized us to replace the original DjVu(r) Reference 
-//C- Library notice by the following text (see doc/lizard2002.djvu):
+//C- DjVuLibre-3.5 is derived from the DjVu(r) Reference Library from
+//C- Lizardtech Software.  Lizardtech Software has authorized us to
+//C- replace the original DjVu(r) Reference Library notice by the following
+//C- text (see doc/lizard2002.djvu and doc/lizardtech2007.djvu):
 //C-
 //C-  ------------------------------------------------------------------
 //C- | DjVu (r) Reference Library (v. 3.5)
@@ -26,7 +27,8 @@
 //C- | 6,058,214 and patents pending.
 //C- |
 //C- | This software is subject to, and may be distributed under, the
-//C- | GNU General Public License, Version 2. The license should have
+//C- | GNU General Public License, either Version 2 of the license,
+//C- | or (at your option) any later version. The license should have
 //C- | accompanied the software or you may obtain a copy of the license
 //C- | from the Free Software Foundation at http://www.fsf.org .
 //C- |
@@ -50,9 +52,7 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-//C- */ 
-
-/* $Id: ddjvuapi.h,v 1.60 2006/12/12 05:58:46 leonb Exp $ */
+*/
 
 #ifndef DDJVUAPI_H
 #define DDJVUAPI_H
@@ -67,12 +67,23 @@ extern "C" {
 #include <stdlib.h>
 #include <stdio.h>
 
+#ifndef DDJVUAPI
+# ifdef WIN32
+#  ifdef DLL_EXPORT
+#   define DDJVUAPI __declspec(dllexport)
+#  else
+#   define DDJVUAPI __declspec(dllimport)
+#  endif
+# endif
+#endif
 #ifndef DDJVUAPI
 # define DDJVUAPI /**/
 #endif
+
 #ifndef TRUE
 # define TRUE  (1)
 #endif
+
 #ifndef FALSE
 # define FALSE (0)
 #endif
@@ -99,6 +110,12 @@ extern "C" {
 
    Version   Change
    -----------------------------
+     20    Added:
+              ddjvu_get_version_string()
+              ddjvu_format_set_white()
+              ddjvu_anno_get_xmp()
+     19    Added:
+              ddjvu_document_create_by_filename_utf8()
      18    Added:
               ddjvu_document_get_{anno,pagedump,filedump}()
            Modifed (binary compatible):
@@ -120,9 +137,7 @@ extern "C" {
      14    Initial version.
 */
 
-#define DDJVUAPI_VERSION 18
-
-
+#define DDJVUAPI_VERSION 20
 
 typedef struct ddjvu_context_s    ddjvu_context_t;
 typedef union  ddjvu_message_s    ddjvu_message_t;
@@ -170,6 +185,14 @@ typedef struct ddjvu_rectmapper_s ddjvu_rectmapper_t;
  */
 
 
+
+/* ddjvu_get_version_string() ---
+   Returns a string that described the underlying code. */
+
+DDJVUAPI const char*
+ddjvu_get_version_string(void);
+
+
 /* ddjvu_context_create ---
    Creates a <ddjvu_context_t> object.
    Argument <programname> is the name of the calling executable. */
@@ -189,6 +212,8 @@ ddjvu_context_release(ddjvu_context_t *context);
 
 
 
+
+
 /* ------- CACHE ------- */
 
 /* ddjvu_cache_set_size ---
@@ -339,7 +364,7 @@ ddjvu_job_stop(ddjvu_job_t *job);
 
 /* ddjvu_job_set_user_data ---
    ddjvu_job_get_user_data ---
-   Each job can store an arbitray pointer
+   Each job can store an arbitrary pointer
    that callers can use for any purpose. These two 
    functions provide for accessing or setting this pointer. 
    This pointer is cleared when the job is released */
@@ -476,8 +501,8 @@ struct ddjvu_message_info_s {   /* ddjvu_message_t::m_info */
    and <ddjvu_stream_close>.
 
    Localized characters in argument <url> should be in 
-   urlencoded utf-8 (like "%2A"). What is happening for non 
-   ascii characters is unclear (probably plain utf8). */
+   urlencoded UTF-8 (like "%2A"). What is happening for non 
+   ascii characters is unclear (probably UTF-8). */
 
 DDJVUAPI ddjvu_document_t *
 ddjvu_document_create(ddjvu_context_t *context,
@@ -488,13 +513,19 @@ ddjvu_document_create(ddjvu_context_t *context,
 /* ddjvu_document_create_by_filename ---
    Creates a document for a DjVu document stored in a file.
    The document will directly access the specified DjVu file 
-   or related files without generating <m_newstream> messages. */
+   or related files without generating <m_newstream> messages.
+   The standard function expects the filename in locale encoding. 
+   The utf8 variant expects an utf8 encoded filename. */
 
 DDJVUAPI ddjvu_document_t *
 ddjvu_document_create_by_filename(ddjvu_context_t *context,
                                   const char *filename,
                                   int cache);
 
+DDJVUAPI ddjvu_document_t *
+ddjvu_document_create_by_filename_utf8(ddjvu_context_t *context,
+                                       const char *filename,
+                                       int cache);
 
 /* ddjvu_document_job ---
    Access the job object in charge of decoding the document header. 
@@ -559,15 +590,15 @@ ddjvu_document_job(ddjvu_document_t *document);
    object is created.
 
    Further <m_newstream> messages are generated to access
-   the auxilliary files of indirect or indexed DjVu
+   the auxiliary files of indirect or indexed DjVu
    documents.  Member <name> then provides the basename of
-   the auxilliary file.
+   the auxiliary file.
 
    Member <url> is set according to the url argument
    provided to function <ddjvu_document_create>.  The first
    newstream message always contain the url passed to
    <ddjvu_document_create>.  Subsequent newstream messages
-   contain the url of the auxilliary files for indirect or
+   contain the url of the auxiliary files for indirect or
    indexed DjVu documents. */
    
 struct ddjvu_message_newstream_s { /* ddjvu_message_t::m_newstream */
@@ -908,7 +939,7 @@ struct ddjvu_message_chunk_s {     /* ddjvu_message_t::m_chunk */
    When receiving <m_relayout>, the viewer should get the image size, decide
    zoom factors, and place the image area, scrollbars, toolbars, and other gui
    objects.  When receiving <m_redisplay>, the viewer should invalidate the
-   image area so that the gui toolkint calls the repaint event handler. This
+   image area so that the gui toolkit calls the repaint event handler. This
    handler should call ddjvu_page_render() and paint the part of the
    image that needs repainting. */
 
@@ -932,7 +963,7 @@ DDJVUAPI int
 ddjvu_page_get_height(ddjvu_page_t *page);
 
 /* ddjvu_page_get_resolution ---
-   Returns the page resolution in pixels per inche (dpi).
+   Returns the page resolution in pixels per inch  (dpi).
    Calling this function before receiving a <m_pageinfo>
    message yields a meaningless but plausible value. */
 
@@ -966,6 +997,7 @@ ddjvu_page_get_version(ddjvu_page_t *page);
 DDJVUAPI int
 ddjvu_code_get_version(void);
 
+
 /* ddjvu_page_get_type ---
    Returns the type of the page data.
    Calling this function before the termination of the
@@ -1230,6 +1262,16 @@ DDJVUAPI void
 ddjvu_format_set_gamma(ddjvu_format_t *format, double gamma);
 
 
+/* ddjvu_format_set_white ---
+   Sets the whitepoint of the display for which the pixels are
+   intended.  This will be combined with the gamma stored in
+   DjVu documents in order to compute a suitable color
+   correction.  The default value is 0xff,0xff,0xff. */
+
+DDJVUAPI void
+ddjvu_format_set_white(ddjvu_format_t *format, 
+                       unsigned char b, unsigned char g, unsigned char r);
+
 /* ddjvu_format_release ---
    Release a reference to a <ddjvu_format_t> object.
    The calling program should no longer reference this object. */
@@ -1337,7 +1379,8 @@ ddjvu_document_print(ddjvu_document_t *document, FILE *output,
      Reordering or duplicating pages is prohibited. 
    * Option "-indirect=<filename>" causes the creation
      of an indirect document with index file <filename>
-     and auxilliary files in the same directory.
+     and auxiliary files in the same directory.
+     The file name is UTF-8 encoded.
      When this option is specified, the argument <output>
      is ignored and should be NULL.
 */
@@ -1415,13 +1458,18 @@ ddjvu_document_get_outline(ddjvu_document_t *document);
 /* ddjvu_document_get_anno --
    This function returns the document-wide annotations.
    This corresponds to a proposed change in the djvu format.
-   This function returns <miniexp_dummy> is the information
-   is not yet available. It may then cause the emission 
-   of <m_pageinfo> messages with null <m_any.page>.
    When no new-style document-wide annotations are available
    and <compat> is true, this function searches a shared 
    annotation chunk and returns its contents.
-*/   
+
+   This function returns <miniexp_dummy> if the information
+   is not yet available. It may then cause the emission 
+   of <m_pageinfo> messages with null <m_any.page>.
+
+   This function returns the empty list <miniexp_nil> when
+   the document does not contain page annotations. It can also
+   return symbols <failed> or <stopped> when an error occurs
+   while accessing the desired information. */
 
 DDJVUAPI miniexp_t
 ddjvu_document_get_anno(ddjvu_document_t *document, int compat);
@@ -1447,8 +1495,8 @@ ddjvu_document_get_anno(ddjvu_document_t *document, int compat);
    accessing the desired information. 
 
    Argument <maxdetail> controls the level of detail in the
-   returned s-expression. Values "page", "region", "para", 
-   or "line" restrict the output to the specified granularity.
+   returned s-expression. Values "page", "column", "region", "para", 
+   "line", and "word" restrict the output to the specified granularity.
    All other values produce a s-expression that represents
    the hidden text data as finely as possible. */
 
@@ -1533,7 +1581,7 @@ ddjvu_anno_get_vertalign(miniexp_t annotations);
 /* ddjvu_anno_get_hyperlinks --
    Parse the annotations and returns a zero terminated 
    array of <(maparea ...)> s-expressions.
-   The called should free this array with function <free>.
+   The caller should free this array with function <free>.
    These s-expressions remain allocated as long
    as the annotations remain allocated.
    See also <(maparea ...)> in the djvused man page. */
@@ -1545,7 +1593,7 @@ ddjvu_anno_get_hyperlinks(miniexp_t annotations);
 /* ddjvu_anno_get_metadata_keys --
    Parse the annotations and returns a zero terminated 
    array of key symbols for the page metadata.
-   The called should free this array with function <free>.
+   The caller should free this array with function <free>.
    See also <(metadata ...)> in the djvused man page. */
 
 DDJVUAPI miniexp_t *
@@ -1563,6 +1611,15 @@ DDJVUAPI const char *
 ddjvu_anno_get_metadata(miniexp_t annotations, miniexp_t key);
 
 
+/* ddjvu_anno_get_xmp --
+   Parse the annotations and returns the xmp metadata string.
+   The string remains allocated as long as the 
+   annotations s-expression remain allocated.
+   Returns zero if no such key is present. */
+
+DDJVUAPI const char *
+ddjvu_anno_get_xmp(miniexp_t annotations, miniexp_t xmp);
+
 
 /* -------------------------------------------------- */
 /* DJVU_MESSAGE_T                                     */
